#ifndef vtk_netcdf_mangle_h
#define vtk_netcdf_mangle_h

#define add_to_NCList vtknetcdf_add_to_NCList
#define atomic_name vtknetcdf_atomic_name
#define cdChar2Comp vtknetcdf_cdChar2Comp
#define Cde2h vtknetcdf_Cde2h
#define Cdh2e vtknetcdf_Cdh2e
#define cdParseRelunits vtknetcdf_cdParseRelunits
#define cdRel2Iso vtknetcdf_cdRel2Iso
#define cdSetErrOpts vtknetcdf_cdSetErrOpts
#define count_NCList vtknetcdf_count_NCList
#define del_from_NCList vtknetcdf_del_from_NCList
#define dup_NC_attrarrayV vtknetcdf_dup_NC_attrarrayV
#define dup_NC_dimarrayV vtknetcdf_dup_NC_dimarrayV
#define dup_NC_vararrayV vtknetcdf_dup_NC_vararrayV
#define elem_NC_attrarray vtknetcdf_elem_NC_attrarray
#define elem_NC_dimarray vtknetcdf_elem_NC_dimarray
#define fill_NC_var vtknetcdf_fill_NC_var
#define find_in_NCList vtknetcdf_find_in_NCList
#define find_in_NCList_by_name vtknetcdf_find_in_NCList_by_name
#define find_NC_Udim vtknetcdf_find_NC_Udim
#define free_NC vtknetcdf_free_NC
#define free_NC_attr vtknetcdf_free_NC_attr
#define free_NC_attrarrayV vtknetcdf_free_NC_attrarrayV
#define free_NC_attrarrayV0 vtknetcdf_free_NC_attrarrayV0
#define free_NC_dim vtknetcdf_free_NC_dim
#define free_NC_dimarrayV vtknetcdf_free_NC_dimarrayV
#define free_NC_dimarrayV0 vtknetcdf_free_NC_dimarrayV0
#define free_NCList vtknetcdf_free_NCList
#define free_NC_string vtknetcdf_free_NC_string
#define free_NC_var vtknetcdf_free_NC_var
#define free_NC_vararrayV vtknetcdf_free_NC_vararrayV
#define free_NC_vararrayV0 vtknetcdf_free_NC_vararrayV0
#define hash_fast vtknetcdf_hash_fast
#define int_cmp vtknetcdf_int_cmp
#define iterate_NCList vtknetcdf_iterate_NCList
#define memio_create vtknetcdf_memio_create
#define memio_open vtknetcdf_memio_open
#define NC3_abort vtknetcdf_NC3_abort
#define nc3_cktype vtknetcdf_nc3_cktype
#define NC3_close vtknetcdf_NC3_close
#define NC3_create vtknetcdf_NC3_create
#define NC3_def_dim vtknetcdf_NC3_def_dim
#define NC3_def_var vtknetcdf_NC3_def_var
#define NC3_del_att vtknetcdf_NC3_del_att
#define NC3__enddef vtknetcdf_NC3__enddef
#define NC3_finalize vtknetcdf_NC3_finalize
#define NC3_get_att vtknetcdf_NC3_get_att
#define NC3_get_vara vtknetcdf_NC3_get_vara
#define NC3_initialize vtknetcdf_NC3_initialize
#define NC3_inq vtknetcdf_NC3_inq
#define NC3_inq_att vtknetcdf_NC3_inq_att
#define NC3_inq_attid vtknetcdf_NC3_inq_attid
#define NC3_inq_attname vtknetcdf_NC3_inq_attname
#define NC3_inq_base_pe vtknetcdf_NC3_inq_base_pe
#define NC3_inq_default_fill_value vtknetcdf_NC3_inq_default_fill_value
#define NC3_inq_dim vtknetcdf_NC3_inq_dim
#define NC3_inq_dimid vtknetcdf_NC3_inq_dimid
#define NC3_inq_format vtknetcdf_NC3_inq_format
#define NC3_inq_format_extended vtknetcdf_NC3_inq_format_extended
#define NC3_inq_type vtknetcdf_NC3_inq_type
#define NC3_inq_unlimdim vtknetcdf_NC3_inq_unlimdim
#define NC3_inq_var vtknetcdf_NC3_inq_var
#define NC3_inq_var_fill vtknetcdf_NC3_inq_var_fill
#define NC3_inq_varid vtknetcdf_NC3_inq_varid
#define NC3_open vtknetcdf_NC3_open
#define NC3_put_att vtknetcdf_NC3_put_att
#define NC3_put_vara vtknetcdf_NC3_put_vara
#define NC3_redef vtknetcdf_NC3_redef
#define NC3_rename_att vtknetcdf_NC3_rename_att
#define NC3_rename_dim vtknetcdf_NC3_rename_dim
#define NC3_rename_var vtknetcdf_NC3_rename_var
#define NC3_set_base_pe vtknetcdf_NC3_set_base_pe
#define NC3_set_fill vtknetcdf_NC3_set_fill
#define NC3_sync vtknetcdf_NC3_sync
#define NC4_abort vtknetcdf_NC4_abort
#define nc4_adjust_var_cache vtknetcdf_nc4_adjust_var_cache
#define nc4_att_list_add vtknetcdf_nc4_att_list_add
#define nc4_att_list_del vtknetcdf_nc4_att_list_del
#define nc4_break_coord_var vtknetcdf_nc4_break_coord_var
#define NC4_buildpropinfo vtknetcdf_NC4_buildpropinfo
#define nc4_check_dup_name vtknetcdf_nc4_check_dup_name
#define nc4_check_name vtknetcdf_nc4_check_name
#define nc4_chunk_cache_nelems vtknetcdf_nc4_chunk_cache_nelems
#define nc4_chunk_cache_preemption vtknetcdf_nc4_chunk_cache_preemption
#define nc4_chunk_cache_size vtknetcdf_nc4_chunk_cache_size
#define NC4_close vtknetcdf_NC4_close
#define nc4_convert_type vtknetcdf_nc4_convert_type
#define NC4_create vtknetcdf_NC4_create
#define NC4_def_compound vtknetcdf_NC4_def_compound
#define NC4_def_dim vtknetcdf_NC4_def_dim
#define NC4_def_enum vtknetcdf_NC4_def_enum
#define NC4_def_grp vtknetcdf_NC4_def_grp
#define NC4_def_opaque vtknetcdf_NC4_def_opaque
#define NC4_def_var vtknetcdf_NC4_def_var
#define NC4_def_var_chunking vtknetcdf_NC4_def_var_chunking
#define NC4_def_var_deflate vtknetcdf_NC4_def_var_deflate
#define NC4_def_var_endian vtknetcdf_NC4_def_var_endian
#define NC4_def_var_fill vtknetcdf_NC4_def_var_fill
#define NC4_def_var_fletcher32 vtknetcdf_NC4_def_var_fletcher32
#define NC4_def_vlen vtknetcdf_NC4_def_vlen
#define NC4_del_att vtknetcdf_NC4_del_att
#define nc4_dim_list_add vtknetcdf_nc4_dim_list_add
#define nc4_dim_list_del vtknetcdf_nc4_dim_list_del
#define NC4__enddef vtknetcdf_NC4__enddef
#define nc4_enddef_netcdf4_file vtknetcdf_nc4_enddef_netcdf4_file
#define nc4_enum_member_add vtknetcdf_nc4_enum_member_add
#define nc4_field_list_add vtknetcdf_nc4_field_list_add
#define NC4_fileinfo_init vtknetcdf_NC4_fileinfo_init
#define NC4_finalize vtknetcdf_NC4_finalize
#define nc4_find_dim vtknetcdf_nc4_find_dim
#define nc4_find_dim_len vtknetcdf_nc4_find_dim_len
#define nc4_find_grp_att vtknetcdf_nc4_find_grp_att
#define nc4_find_grp_h5 vtknetcdf_nc4_find_grp_h5
#define nc4_find_g_var_nc vtknetcdf_nc4_find_g_var_nc
#define nc4_find_nc4_grp vtknetcdf_nc4_find_nc4_grp
#define nc4_find_nc_att vtknetcdf_nc4_find_nc_att
#define nc4_find_nc_file vtknetcdf_nc4_find_nc_file
#define nc4_find_nc_grp_h5 vtknetcdf_nc4_find_nc_grp_h5
#define nc4_find_type vtknetcdf_nc4_find_type
#define nc4_find_var vtknetcdf_nc4_find_var
#define nc4_get_att vtknetcdf_nc4_get_att
#define NC4_get_att vtknetcdf_NC4_get_att
#define nc4_get_att_tc vtknetcdf_nc4_get_att_tc
#define nc4_get_default_fill_value vtknetcdf_nc4_get_default_fill_value
#define NC4_get_fileinfo vtknetcdf_NC4_get_fileinfo
#define nc4_get_hdf_typeid vtknetcdf_nc4_get_hdf_typeid
#define nc4_get_typeclass vtknetcdf_nc4_get_typeclass
#define nc4_get_typelen_mem vtknetcdf_nc4_get_typelen_mem
#define nc4_get_vara vtknetcdf_nc4_get_vara
#define NC4_get_vara vtknetcdf_NC4_get_vara
#define NC4_get_var_chunk_cache vtknetcdf_NC4_get_var_chunk_cache
#define NC4_get_vlen_element vtknetcdf_NC4_get_vlen_element
#define nc4_grp_list_add vtknetcdf_nc4_grp_list_add
#define NC4_hdf5get_libversion vtknetcdf_NC4_hdf5get_libversion
#define NC4_hdf5get_superblock vtknetcdf_NC4_hdf5get_superblock
#define nc4_hdf5_initialize vtknetcdf_nc4_hdf5_initialize
#define NC4_initialize vtknetcdf_NC4_initialize
#define NC4_inq vtknetcdf_NC4_inq
#define NC4_inq_att vtknetcdf_NC4_inq_att
#define NC4_inq_attid vtknetcdf_NC4_inq_attid
#define NC4_inq_attname vtknetcdf_NC4_inq_attname
#define NC4_inq_base_pe vtknetcdf_NC4_inq_base_pe
#define NC4_inq_compound_field vtknetcdf_NC4_inq_compound_field
#define NC4_inq_compound_fieldindex vtknetcdf_NC4_inq_compound_fieldindex
#define NC4_inq_dim vtknetcdf_NC4_inq_dim
#define NC4_inq_dimid vtknetcdf_NC4_inq_dimid
#define NC4_inq_dimids vtknetcdf_NC4_inq_dimids
#define NC4_inq_enum_ident vtknetcdf_NC4_inq_enum_ident
#define NC4_inq_enum_member vtknetcdf_NC4_inq_enum_member
#define NC4_inq_format vtknetcdf_NC4_inq_format
#define NC4_inq_format_extended vtknetcdf_NC4_inq_format_extended
#define NC4_inq_grp_full_ncid vtknetcdf_NC4_inq_grp_full_ncid
#define NC4_inq_grpname vtknetcdf_NC4_inq_grpname
#define NC4_inq_grpname_full vtknetcdf_NC4_inq_grpname_full
#define NC4_inq_grp_parent vtknetcdf_NC4_inq_grp_parent
#define NC4_inq_grps vtknetcdf_NC4_inq_grps
#define NC4_inq_ncid vtknetcdf_NC4_inq_ncid
#define NC4_inq_type vtknetcdf_NC4_inq_type
#define NC4_inq_type_equal vtknetcdf_NC4_inq_type_equal
#define NC4_inq_typeid vtknetcdf_NC4_inq_typeid
#define NC4_inq_typeids vtknetcdf_NC4_inq_typeids
#define NC4_inq_unlimdim vtknetcdf_NC4_inq_unlimdim
#define NC4_inq_unlimdims vtknetcdf_NC4_inq_unlimdims
#define NC4_inq_user_type vtknetcdf_NC4_inq_user_type
#define NC4_inq_var_all vtknetcdf_NC4_inq_var_all
#define NC4_inq_varid vtknetcdf_NC4_inq_varid
#define NC4_inq_varids vtknetcdf_NC4_inq_varids
#define NC4_insert_array_compound vtknetcdf_NC4_insert_array_compound
#define NC4_insert_compound vtknetcdf_NC4_insert_compound
#define NC4_insert_enum vtknetcdf_NC4_insert_enum
#define NC4_isnetcdf4 vtknetcdf_NC4_isnetcdf4
#define nc4_nc4f_list_add vtknetcdf_nc4_nc4f_list_add
#define nc4_normalize_name vtknetcdf_nc4_normalize_name
#define NC4_open vtknetcdf_NC4_open
#define nc4_open_var_grp2 vtknetcdf_nc4_open_var_grp2
#define NC4print vtknetcdf_NC4print
#define NC4_put_att vtknetcdf_NC4_put_att
#define NC4_put_propattr vtknetcdf_NC4_put_propattr
#define nc4_put_vara vtknetcdf_nc4_put_vara
#define NC4_put_vara vtknetcdf_NC4_put_vara
#define NC4_put_vlen_element vtknetcdf_NC4_put_vlen_element
#define nc4_rec_detect_need_to_preserve_dimids vtknetcdf_nc4_rec_detect_need_to_preserve_dimids
#define nc4_rec_find_grp vtknetcdf_nc4_rec_find_grp
#define nc4_rec_find_hdf_type vtknetcdf_nc4_rec_find_hdf_type
#define nc4_rec_find_named_type vtknetcdf_nc4_rec_find_named_type
#define nc4_rec_find_nc_type vtknetcdf_nc4_rec_find_nc_type
#define nc4_rec_grp_del vtknetcdf_nc4_rec_grp_del
#define nc4_rec_match_dimscales vtknetcdf_nc4_rec_match_dimscales
#define nc4_rec_write_groups_types vtknetcdf_nc4_rec_write_groups_types
#define nc4_rec_write_metadata vtknetcdf_nc4_rec_write_metadata
#define NC4_redef vtknetcdf_NC4_redef
#define nc4_reform_coord_var vtknetcdf_nc4_reform_coord_var
#define NC4_rename_att vtknetcdf_NC4_rename_att
#define NC4_rename_dim vtknetcdf_NC4_rename_dim
#define NC4_rename_grp vtknetcdf_NC4_rename_grp
#define NC4_rename_var vtknetcdf_NC4_rename_var
#define nc4_reopen_dataset vtknetcdf_nc4_reopen_dataset
#define NC4_set_base_pe vtknetcdf_NC4_set_base_pe
#define NC4_set_fill vtknetcdf_NC4_set_fill
#define NC4_set_var_chunk_cache vtknetcdf_NC4_set_var_chunk_cache
#define NC4_show_metadata vtknetcdf_NC4_show_metadata
#define NC4_sync vtknetcdf_NC4_sync
#define NC4_test_netcdf4 vtknetcdf_NC4_test_netcdf4
#define nc4_type_free vtknetcdf_nc4_type_free
#define nc4typelen vtknetcdf_nc4typelen
#define nc4_type_list_add vtknetcdf_nc4_type_list_add
#define nc4_var_add vtknetcdf_nc4_var_add
#define nc4_vararray_add vtknetcdf_nc4_vararray_add
#define nc4_var_del vtknetcdf_nc4_var_del
#define NC4_var_par_access vtknetcdf_NC4_var_par_access
#define nc4verify vtknetcdf_nc4verify
#define nc_abort vtknetcdf_nc_abort
#define NC_argc vtknetcdf_NC_argc
#define NC_argv vtknetcdf_NC_argv
#define NC_atomictypelen vtknetcdf_NC_atomictypelen
#define NC_atomictypename vtknetcdf_NC_atomictypename
#define ncaux_abort_compound vtknetcdf_ncaux_abort_compound
#define ncaux_add_field vtknetcdf_ncaux_add_field
#define ncaux_begin_compound vtknetcdf_ncaux_begin_compound
#define ncaux_end_compound vtknetcdf_ncaux_end_compound
#define ncbytesappend vtknetcdf_ncbytesappend
#define ncbytesappendn vtknetcdf_ncbytesappendn
#define ncbytescat vtknetcdf_ncbytescat
#define ncbytesdup vtknetcdf_ncbytesdup
#define ncbytesextract vtknetcdf_ncbytesextract
#define ncbytesfill vtknetcdf_ncbytesfill
#define ncbytesfree vtknetcdf_ncbytesfree
#define ncbytesget vtknetcdf_ncbytesget
#define ncbytesnew vtknetcdf_ncbytesnew
#define ncbytesnull vtknetcdf_ncbytesnull
#define ncbytesprepend vtknetcdf_ncbytesprepend
#define ncbytesremove vtknetcdf_ncbytesremove
#define ncbytesset vtknetcdf_ncbytesset
#define ncbytessetalloc vtknetcdf_ncbytessetalloc
#define ncbytessetcontents vtknetcdf_ncbytessetcontents
#define ncbytessetlength vtknetcdf_ncbytessetlength
#define NC_calcsize vtknetcdf_NC_calcsize
#define NC_check_id vtknetcdf_NC_check_id
#define NC_check_name vtknetcdf_NC_check_name
#define NC_check_vlen vtknetcdf_NC_check_vlen
#define NC_check_vlens vtknetcdf_NC_check_vlens
#define nc_close vtknetcdf_nc_close
#define nc_copy_att vtknetcdf_nc_copy_att
#define nc_copy_var vtknetcdf_nc_copy_var
#define nc_create vtknetcdf_nc_create
#define nc__create vtknetcdf_nc__create
#define NC_create vtknetcdf_NC_create
#define nc__create_mp vtknetcdf_nc__create_mp
#define nc_create_par vtknetcdf_nc_create_par
#define nc_create_par_fortran vtknetcdf_nc_create_par_fortran
#define NCDEFAULT_get_varm vtknetcdf_NCDEFAULT_get_varm
#define NCDEFAULT_get_vars vtknetcdf_NCDEFAULT_get_vars
#define NCDEFAULT_put_varm vtknetcdf_NCDEFAULT_put_varm
#define NCDEFAULT_put_vars vtknetcdf_NCDEFAULT_put_vars
#define nc_def_compound vtknetcdf_nc_def_compound
#define nc_def_dim vtknetcdf_nc_def_dim
#define nc_def_enum vtknetcdf_nc_def_enum
#define nc_def_grp vtknetcdf_nc_def_grp
#define nc_def_opaque vtknetcdf_nc_def_opaque
#define nc_def_var vtknetcdf_nc_def_var
#define nc_def_var_chunking vtknetcdf_nc_def_var_chunking
#define nc_def_var_chunking_ints vtknetcdf_nc_def_var_chunking_ints
#define nc_def_var_deflate vtknetcdf_nc_def_var_deflate
#define nc_def_var_endian vtknetcdf_nc_def_var_endian
#define nc_def_var_fill vtknetcdf_nc_def_var_fill
#define nc_def_var_fletcher32 vtknetcdf_nc_def_var_fletcher32
#define nc_def_vlen vtknetcdf_nc_def_vlen
#define nc_del_att vtknetcdf_nc_del_att
#define nc_delete vtknetcdf_nc_delete
#define nc_delete_mp vtknetcdf_nc_delete_mp
#define NCDISPATCH_finalize vtknetcdf_NCDISPATCH_finalize
#define NCDISPATCH_get_att vtknetcdf_NCDISPATCH_get_att
#define NCDISPATCH_initialize vtknetcdf_NCDISPATCH_initialize
#define NCDISPATCH_inq_var_all vtknetcdf_NCDISPATCH_inq_var_all
#define nc_enddef vtknetcdf_nc_enddef
#define nc__enddef vtknetcdf_nc__enddef
#define nc_finalize vtknetcdf_nc_finalize
#define NC_finalized vtknetcdf_NC_finalized
#define NC_findattr vtknetcdf_NC_findattr
#define NC_findvar vtknetcdf_NC_findvar
#define nc_free_string vtknetcdf_nc_free_string
#define nc_free_vlen vtknetcdf_nc_free_vlen
#define nc_free_vlens vtknetcdf_nc_free_vlens
#define nc_get_att vtknetcdf_nc_get_att
#define nc_get_att_double vtknetcdf_nc_get_att_double
#define nc_get_att_float vtknetcdf_nc_get_att_float
#define nc_get_att_int vtknetcdf_nc_get_att_int
#define nc_get_att_long vtknetcdf_nc_get_att_long
#define nc_get_att_longlong vtknetcdf_nc_get_att_longlong
#define nc_get_att_schar vtknetcdf_nc_get_att_schar
#define nc_get_att_short vtknetcdf_nc_get_att_short
#define nc_get_att_string vtknetcdf_nc_get_att_string
#define nc_get_att_text vtknetcdf_nc_get_att_text
#define nc_get_att_ubyte vtknetcdf_nc_get_att_ubyte
#define nc_get_att_uchar vtknetcdf_nc_get_att_uchar
#define nc_get_att_uint vtknetcdf_nc_get_att_uint
#define nc_get_att_ulonglong vtknetcdf_nc_get_att_ulonglong
#define nc_get_att_ushort vtknetcdf_nc_get_att_ushort
#define nc_get_chunk_cache vtknetcdf_nc_get_chunk_cache
#define nc_get_chunk_cache_ints vtknetcdf_nc_get_chunk_cache_ints
#define nc_get_default_format vtknetcdf_nc_get_default_format
#define nc_get_NC vtknetcdf_nc_get_NC
#define NC_getshape vtknetcdf_NC_getshape
#define nc_get_var vtknetcdf_nc_get_var
#define nc_get_var1 vtknetcdf_nc_get_var1
#define nc_get_var1_double vtknetcdf_nc_get_var1_double
#define nc_get_var1_float vtknetcdf_nc_get_var1_float
#define nc_get_var1_int vtknetcdf_nc_get_var1_int
#define nc_get_var1_long vtknetcdf_nc_get_var1_long
#define nc_get_var1_longlong vtknetcdf_nc_get_var1_longlong
#define nc_get_var1_schar vtknetcdf_nc_get_var1_schar
#define nc_get_var1_short vtknetcdf_nc_get_var1_short
#define nc_get_var1_string vtknetcdf_nc_get_var1_string
#define nc_get_var1_text vtknetcdf_nc_get_var1_text
#define nc_get_var1_ubyte vtknetcdf_nc_get_var1_ubyte
#define nc_get_var1_uchar vtknetcdf_nc_get_var1_uchar
#define nc_get_var1_uint vtknetcdf_nc_get_var1_uint
#define nc_get_var1_ulonglong vtknetcdf_nc_get_var1_ulonglong
#define nc_get_var1_ushort vtknetcdf_nc_get_var1_ushort
#define nc_get_vara vtknetcdf_nc_get_vara
#define NC_get_vara vtknetcdf_NC_get_vara
#define nc_get_vara_double vtknetcdf_nc_get_vara_double
#define nc_get_vara_float vtknetcdf_nc_get_vara_float
#define nc_get_vara_int vtknetcdf_nc_get_vara_int
#define nc_get_vara_long vtknetcdf_nc_get_vara_long
#define nc_get_vara_longlong vtknetcdf_nc_get_vara_longlong
#define nc_get_vara_schar vtknetcdf_nc_get_vara_schar
#define nc_get_vara_short vtknetcdf_nc_get_vara_short
#define nc_get_vara_string vtknetcdf_nc_get_vara_string
#define nc_get_vara_text vtknetcdf_nc_get_vara_text
#define nc_get_vara_ubyte vtknetcdf_nc_get_vara_ubyte
#define nc_get_vara_uchar vtknetcdf_nc_get_vara_uchar
#define nc_get_vara_uint vtknetcdf_nc_get_vara_uint
#define nc_get_vara_ulonglong vtknetcdf_nc_get_vara_ulonglong
#define nc_get_vara_ushort vtknetcdf_nc_get_vara_ushort
#define nc_get_var_chunk_cache vtknetcdf_nc_get_var_chunk_cache
#define nc_get_var_chunk_cache_ints vtknetcdf_nc_get_var_chunk_cache_ints
#define nc_get_var_double vtknetcdf_nc_get_var_double
#define nc_get_var_float vtknetcdf_nc_get_var_float
#define nc_get_var_int vtknetcdf_nc_get_var_int
#define nc_get_var_long vtknetcdf_nc_get_var_long
#define nc_get_var_longlong vtknetcdf_nc_get_var_longlong
#define nc_get_varm vtknetcdf_nc_get_varm
#define nc_get_varm_double vtknetcdf_nc_get_varm_double
#define nc_get_varm_float vtknetcdf_nc_get_varm_float
#define nc_get_varm_int vtknetcdf_nc_get_varm_int
#define nc_get_varm_long vtknetcdf_nc_get_varm_long
#define nc_get_varm_longlong vtknetcdf_nc_get_varm_longlong
#define nc_get_varm_schar vtknetcdf_nc_get_varm_schar
#define nc_get_varm_short vtknetcdf_nc_get_varm_short
#define nc_get_varm_string vtknetcdf_nc_get_varm_string
#define nc_get_varm_text vtknetcdf_nc_get_varm_text
#define nc_get_varm_ubyte vtknetcdf_nc_get_varm_ubyte
#define nc_get_varm_uchar vtknetcdf_nc_get_varm_uchar
#define nc_get_varm_uint vtknetcdf_nc_get_varm_uint
#define nc_get_varm_ulonglong vtknetcdf_nc_get_varm_ulonglong
#define nc_get_varm_ushort vtknetcdf_nc_get_varm_ushort
#define nc_get_vars vtknetcdf_nc_get_vars
#define nc_get_var_schar vtknetcdf_nc_get_var_schar
#define nc_get_vars_double vtknetcdf_nc_get_vars_double
#define nc_get_vars_float vtknetcdf_nc_get_vars_float
#define nc_get_var_short vtknetcdf_nc_get_var_short
#define nc_get_vars_int vtknetcdf_nc_get_vars_int
#define nc_get_vars_long vtknetcdf_nc_get_vars_long
#define nc_get_vars_longlong vtknetcdf_nc_get_vars_longlong
#define nc_get_vars_schar vtknetcdf_nc_get_vars_schar
#define nc_get_vars_short vtknetcdf_nc_get_vars_short
#define nc_get_vars_string vtknetcdf_nc_get_vars_string
#define nc_get_vars_text vtknetcdf_nc_get_vars_text
#define nc_get_var_string vtknetcdf_nc_get_var_string
#define nc_get_vars_ubyte vtknetcdf_nc_get_vars_ubyte
#define nc_get_vars_uchar vtknetcdf_nc_get_vars_uchar
#define nc_get_vars_uint vtknetcdf_nc_get_vars_uint
#define nc_get_vars_ulonglong vtknetcdf_nc_get_vars_ulonglong
#define nc_get_vars_ushort vtknetcdf_nc_get_vars_ushort
#define nc_get_var_text vtknetcdf_nc_get_var_text
#define nc_get_var_ubyte vtknetcdf_nc_get_var_ubyte
#define nc_get_var_uchar vtknetcdf_nc_get_var_uchar
#define nc_get_var_uint vtknetcdf_nc_get_var_uint
#define nc_get_var_ulonglong vtknetcdf_nc_get_var_ulonglong
#define nc_get_var_ushort vtknetcdf_nc_get_var_ushort
#define nc_get_vlen_element vtknetcdf_nc_get_vlen_element
#define nchashfree vtknetcdf_nchashfree
#define nchashget vtknetcdf_nchashget
#define nchashinsert vtknetcdf_nchashinsert
#define nchashith vtknetcdf_nchashith
#define nchashkeys vtknetcdf_nchashkeys
#define nchashlookup vtknetcdf_nchashlookup
#define NC_hashmapAddDim vtknetcdf_NC_hashmapAddDim
#define NC_hashmapAddVar vtknetcdf_NC_hashmapAddVar
#define NC_hashmapCount vtknetcdf_NC_hashmapCount
#define NC_hashmapCreate vtknetcdf_NC_hashmapCreate
#define NC_hashmapDelete vtknetcdf_NC_hashmapDelete
#define NC_hashmapGetDim vtknetcdf_NC_hashmapGetDim
#define NC_hashmapGetVar vtknetcdf_NC_hashmapGetVar
#define NC_hashmapRemoveDim vtknetcdf_NC_hashmapRemoveDim
#define NC_hashmapRemoveVar vtknetcdf_NC_hashmapRemoveVar
#define nchashnew vtknetcdf_nchashnew
#define nchashnew0 vtknetcdf_nchashnew0
#define nchashremove vtknetcdf_nchashremove
#define nchashreplace vtknetcdf_nchashreplace
#define nc_initialize vtknetcdf_nc_initialize
#define nc_inq vtknetcdf_nc_inq
#define nc_inq_att vtknetcdf_nc_inq_att
#define nc_inq_attid vtknetcdf_nc_inq_attid
#define nc_inq_attlen vtknetcdf_nc_inq_attlen
#define nc_inq_attname vtknetcdf_nc_inq_attname
#define nc_inq_atttype vtknetcdf_nc_inq_atttype
#define nc_inq_base_pe vtknetcdf_nc_inq_base_pe
#define nc_inq_compound vtknetcdf_nc_inq_compound
#define nc_inq_compound_field vtknetcdf_nc_inq_compound_field
#define nc_inq_compound_fielddim_sizes vtknetcdf_nc_inq_compound_fielddim_sizes
#define nc_inq_compound_fieldindex vtknetcdf_nc_inq_compound_fieldindex
#define nc_inq_compound_fieldname vtknetcdf_nc_inq_compound_fieldname
#define nc_inq_compound_fieldndims vtknetcdf_nc_inq_compound_fieldndims
#define nc_inq_compound_fieldoffset vtknetcdf_nc_inq_compound_fieldoffset
#define nc_inq_compound_fieldtype vtknetcdf_nc_inq_compound_fieldtype
#define nc_inq_compound_name vtknetcdf_nc_inq_compound_name
#define nc_inq_compound_nfields vtknetcdf_nc_inq_compound_nfields
#define nc_inq_compound_size vtknetcdf_nc_inq_compound_size
#define nc_inq_dim vtknetcdf_nc_inq_dim
#define nc_inq_dimid vtknetcdf_nc_inq_dimid
#define nc_inq_dimids vtknetcdf_nc_inq_dimids
#define nc_inq_dimlen vtknetcdf_nc_inq_dimlen
#define nc_inq_dimname vtknetcdf_nc_inq_dimname
#define nc_inq_enum vtknetcdf_nc_inq_enum
#define nc_inq_enum_ident vtknetcdf_nc_inq_enum_ident
#define nc_inq_enum_member vtknetcdf_nc_inq_enum_member
#define nc_inq_format vtknetcdf_nc_inq_format
#define nc_inq_format_extended vtknetcdf_nc_inq_format_extended
#define nc_inq_grp_full_ncid vtknetcdf_nc_inq_grp_full_ncid
#define nc_inq_grpname vtknetcdf_nc_inq_grpname
#define nc_inq_grpname_full vtknetcdf_nc_inq_grpname_full
#define nc_inq_grpname_len vtknetcdf_nc_inq_grpname_len
#define nc_inq_grp_ncid vtknetcdf_nc_inq_grp_ncid
#define nc_inq_grp_parent vtknetcdf_nc_inq_grp_parent
#define nc_inq_grps vtknetcdf_nc_inq_grps
#define nc_inq_libvers vtknetcdf_nc_inq_libvers
#define nc_inq_natts vtknetcdf_nc_inq_natts
#define nc_inq_ncid vtknetcdf_nc_inq_ncid
#define nc_inq_ndims vtknetcdf_nc_inq_ndims
#define nc_inq_nvars vtknetcdf_nc_inq_nvars
#define nc_inq_opaque vtknetcdf_nc_inq_opaque
#define nc_inq_path vtknetcdf_nc_inq_path
#define NC_inq_recvar vtknetcdf_NC_inq_recvar
#define nc_inq_type vtknetcdf_nc_inq_type
#define nc_inq_type_equal vtknetcdf_nc_inq_type_equal
#define nc_inq_typeid vtknetcdf_nc_inq_typeid
#define nc_inq_typeids vtknetcdf_nc_inq_typeids
#define nc_inq_unlimdim vtknetcdf_nc_inq_unlimdim
#define nc_inq_unlimdims vtknetcdf_nc_inq_unlimdims
#define nc_inq_user_type vtknetcdf_nc_inq_user_type
#define nc_inq_var vtknetcdf_nc_inq_var
#define NC_inq_var_all vtknetcdf_NC_inq_var_all
#define nc_inq_var_chunking vtknetcdf_nc_inq_var_chunking
#define nc_inq_var_chunking_ints vtknetcdf_nc_inq_var_chunking_ints
#define nc_inq_var_deflate vtknetcdf_nc_inq_var_deflate
#define nc_inq_vardimid vtknetcdf_nc_inq_vardimid
#define nc_inq_var_endian vtknetcdf_nc_inq_var_endian
#define nc_inq_var_fill vtknetcdf_nc_inq_var_fill
#define nc_inq_var_fletcher32 vtknetcdf_nc_inq_var_fletcher32
#define nc_inq_varid vtknetcdf_nc_inq_varid
#define nc_inq_varids vtknetcdf_nc_inq_varids
#define nc_inq_varname vtknetcdf_nc_inq_varname
#define nc_inq_varnatts vtknetcdf_nc_inq_varnatts
#define nc_inq_varndims vtknetcdf_nc_inq_varndims
#define nc_inq_var_szip vtknetcdf_nc_inq_var_szip
#define nc_inq_vartype vtknetcdf_nc_inq_vartype
#define nc_inq_vlen vtknetcdf_nc_inq_vlen
#define nc_insert_array_compound vtknetcdf_nc_insert_array_compound
#define nc_insert_compound vtknetcdf_nc_insert_compound
#define nc_insert_enum vtknetcdf_nc_insert_enum
#define ncio_close vtknetcdf_ncio_close
#define ncio_create vtknetcdf_ncio_create
#define ncio_filesize vtknetcdf_ncio_filesize
#define ncio_get vtknetcdf_ncio_get
#define ncio_move vtknetcdf_ncio_move
#define ncio_open vtknetcdf_ncio_open
#define ncio_pad_length vtknetcdf_ncio_pad_length
#define ncio_rel vtknetcdf_ncio_rel
#define ncio_sync vtknetcdf_ncio_sync
#define NC_is_recvar vtknetcdf_NC_is_recvar
#define nclistclone vtknetcdf_nclistclone
#define nclistcontains vtknetcdf_nclistcontains
#define nclistdup vtknetcdf_nclistdup
#define nclistelemremove vtknetcdf_nclistelemremove
#define nclistextract vtknetcdf_nclistextract
#define nclistfree vtknetcdf_nclistfree
#define nclistfreeall vtknetcdf_nclistfreeall
#define nclistget vtknetcdf_nclistget
#define nclistinsert vtknetcdf_nclistinsert
#define nclistnew vtknetcdf_nclistnew
#define nclistnull vtknetcdf_nclistnull
#define nclistpop vtknetcdf_nclistpop
#define nclistpush vtknetcdf_nclistpush
#define nclistremove vtknetcdf_nclistremove
#define nclistset vtknetcdf_nclistset
#define nclistsetalloc vtknetcdf_nclistsetalloc
#define nclistsetlength vtknetcdf_nclistsetlength
#define nclisttop vtknetcdf_nclisttop
#define nclistunique vtknetcdf_nclistunique
#define nclog vtknetcdf_nclog
#define nclogclose vtknetcdf_nclogclose
#define ncloginit vtknetcdf_ncloginit
#define nclogopen vtknetcdf_nclogopen
#define nclogsettags vtknetcdf_nclogsettags
#define nclogtext vtknetcdf_nclogtext
#define nclogtextn vtknetcdf_nclogtextn
#define NC_lookupvar vtknetcdf_NC_lookupvar
#define nc_open vtknetcdf_nc_open
#define nc__open vtknetcdf_nc__open
#define NC_open vtknetcdf_NC_open
#define nc_open_mem vtknetcdf_nc_open_mem
#define nc__open_mp vtknetcdf_nc__open_mp
#define nc_open_par vtknetcdf_nc_open_par
#define nc_open_par_fortran vtknetcdf_nc_open_par_fortran
#define NCpathcvt vtknetcdf_NCpathcvt
#define nc__pseudofd vtknetcdf_nc__pseudofd
#define nc_put_att vtknetcdf_nc_put_att
#define nc_put_att_double vtknetcdf_nc_put_att_double
#define nc_put_att_float vtknetcdf_nc_put_att_float
#define nc_put_att_int vtknetcdf_nc_put_att_int
#define nc_put_att_long vtknetcdf_nc_put_att_long
#define nc_put_att_longlong vtknetcdf_nc_put_att_longlong
#define nc_put_att_schar vtknetcdf_nc_put_att_schar
#define nc_put_att_short vtknetcdf_nc_put_att_short
#define nc_put_att_string vtknetcdf_nc_put_att_string
#define nc_put_att_text vtknetcdf_nc_put_att_text
#define nc_put_att_ubyte vtknetcdf_nc_put_att_ubyte
#define nc_put_att_uchar vtknetcdf_nc_put_att_uchar
#define nc_put_att_uint vtknetcdf_nc_put_att_uint
#define nc_put_att_ulonglong vtknetcdf_nc_put_att_ulonglong
#define nc_put_att_ushort vtknetcdf_nc_put_att_ushort
#define nc_put_var vtknetcdf_nc_put_var
#define nc_put_var1 vtknetcdf_nc_put_var1
#define nc_put_var1_double vtknetcdf_nc_put_var1_double
#define nc_put_var1_float vtknetcdf_nc_put_var1_float
#define nc_put_var1_int vtknetcdf_nc_put_var1_int
#define nc_put_var1_long vtknetcdf_nc_put_var1_long
#define nc_put_var1_longlong vtknetcdf_nc_put_var1_longlong
#define nc_put_var1_schar vtknetcdf_nc_put_var1_schar
#define nc_put_var1_short vtknetcdf_nc_put_var1_short
#define nc_put_var1_string vtknetcdf_nc_put_var1_string
#define nc_put_var1_text vtknetcdf_nc_put_var1_text
#define nc_put_var1_ubyte vtknetcdf_nc_put_var1_ubyte
#define nc_put_var1_uchar vtknetcdf_nc_put_var1_uchar
#define nc_put_var1_uint vtknetcdf_nc_put_var1_uint
#define nc_put_var1_ulonglong vtknetcdf_nc_put_var1_ulonglong
#define nc_put_var1_ushort vtknetcdf_nc_put_var1_ushort
#define nc_put_vara vtknetcdf_nc_put_vara
#define nc_put_vara_double vtknetcdf_nc_put_vara_double
#define nc_put_vara_float vtknetcdf_nc_put_vara_float
#define nc_put_vara_int vtknetcdf_nc_put_vara_int
#define nc_put_vara_long vtknetcdf_nc_put_vara_long
#define nc_put_vara_longlong vtknetcdf_nc_put_vara_longlong
#define nc_put_vara_schar vtknetcdf_nc_put_vara_schar
#define nc_put_vara_short vtknetcdf_nc_put_vara_short
#define nc_put_vara_string vtknetcdf_nc_put_vara_string
#define nc_put_vara_text vtknetcdf_nc_put_vara_text
#define nc_put_vara_ubyte vtknetcdf_nc_put_vara_ubyte
#define nc_put_vara_uchar vtknetcdf_nc_put_vara_uchar
#define nc_put_vara_uint vtknetcdf_nc_put_vara_uint
#define nc_put_vara_ulonglong vtknetcdf_nc_put_vara_ulonglong
#define nc_put_vara_ushort vtknetcdf_nc_put_vara_ushort
#define nc_put_var_double vtknetcdf_nc_put_var_double
#define nc_put_var_float vtknetcdf_nc_put_var_float
#define nc_put_var_int vtknetcdf_nc_put_var_int
#define nc_put_var_long vtknetcdf_nc_put_var_long
#define nc_put_var_longlong vtknetcdf_nc_put_var_longlong
#define nc_put_varm vtknetcdf_nc_put_varm
#define nc_put_varm_double vtknetcdf_nc_put_varm_double
#define nc_put_varm_float vtknetcdf_nc_put_varm_float
#define nc_put_varm_int vtknetcdf_nc_put_varm_int
#define nc_put_varm_long vtknetcdf_nc_put_varm_long
#define nc_put_varm_longlong vtknetcdf_nc_put_varm_longlong
#define nc_put_varm_schar vtknetcdf_nc_put_varm_schar
#define nc_put_varm_short vtknetcdf_nc_put_varm_short
#define nc_put_varm_string vtknetcdf_nc_put_varm_string
#define nc_put_varm_text vtknetcdf_nc_put_varm_text
#define nc_put_varm_ubyte vtknetcdf_nc_put_varm_ubyte
#define nc_put_varm_uchar vtknetcdf_nc_put_varm_uchar
#define nc_put_varm_uint vtknetcdf_nc_put_varm_uint
#define nc_put_varm_ulonglong vtknetcdf_nc_put_varm_ulonglong
#define nc_put_varm_ushort vtknetcdf_nc_put_varm_ushort
#define nc_put_vars vtknetcdf_nc_put_vars
#define nc_put_var_schar vtknetcdf_nc_put_var_schar
#define nc_put_vars_double vtknetcdf_nc_put_vars_double
#define nc_put_vars_float vtknetcdf_nc_put_vars_float
#define nc_put_var_short vtknetcdf_nc_put_var_short
#define nc_put_vars_int vtknetcdf_nc_put_vars_int
#define nc_put_vars_long vtknetcdf_nc_put_vars_long
#define nc_put_vars_longlong vtknetcdf_nc_put_vars_longlong
#define nc_put_vars_schar vtknetcdf_nc_put_vars_schar
#define nc_put_vars_short vtknetcdf_nc_put_vars_short
#define nc_put_vars_string vtknetcdf_nc_put_vars_string
#define nc_put_vars_text vtknetcdf_nc_put_vars_text
#define nc_put_var_string vtknetcdf_nc_put_var_string
#define nc_put_vars_ubyte vtknetcdf_nc_put_vars_ubyte
#define nc_put_vars_uchar vtknetcdf_nc_put_vars_uchar
#define nc_put_vars_uint vtknetcdf_nc_put_vars_uint
#define nc_put_vars_ulonglong vtknetcdf_nc_put_vars_ulonglong
#define nc_put_vars_ushort vtknetcdf_nc_put_vars_ushort
#define nc_put_var_text vtknetcdf_nc_put_var_text
#define nc_put_var_ubyte vtknetcdf_nc_put_var_ubyte
#define nc_put_var_uchar vtknetcdf_nc_put_var_uchar
#define nc_put_var_uint vtknetcdf_nc_put_var_uint
#define nc_put_var_ulonglong vtknetcdf_nc_put_var_ulonglong
#define nc_put_var_ushort vtknetcdf_nc_put_var_ushort
#define nc_put_vlen_element vtknetcdf_nc_put_vlen_element
#define ncrc_load vtknetcdf_ncrc_load
#define ncrc_lookup vtknetcdf_ncrc_lookup
#define ncrc_reset vtknetcdf_ncrc_reset
#define nc_redef vtknetcdf_nc_redef
#define nc_rename_att vtknetcdf_nc_rename_att
#define nc_rename_dim vtknetcdf_nc_rename_dim
#define nc_rename_grp vtknetcdf_nc_rename_grp
#define nc_rename_var vtknetcdf_nc_rename_var
#define NC_RESERVED_ATT_LIST vtknetcdf_NC_RESERVED_ATT_LIST
#define NC_RESERVED_SPECIAL_LIST vtknetcdf_NC_RESERVED_SPECIAL_LIST
#define NC_RESERVED_VARATT_LIST vtknetcdf_NC_RESERVED_VARATT_LIST
#define nc_set_base_pe vtknetcdf_nc_set_base_pe
#define nc_set_chunk_cache vtknetcdf_nc_set_chunk_cache
#define nc_set_chunk_cache_ints vtknetcdf_nc_set_chunk_cache_ints
#define nc_set_default_format vtknetcdf_nc_set_default_format
#define nc_set_fill vtknetcdf_nc_set_fill
#define ncsetlogging vtknetcdf_ncsetlogging
#define nc_set_var_chunk_cache vtknetcdf_nc_set_var_chunk_cache
#define nc_set_var_chunk_cache_ints vtknetcdf_nc_set_var_chunk_cache_ints
#define nc_show_metadata vtknetcdf_nc_show_metadata
#define nc_strerror vtknetcdf_nc_strerror
#define ncstrndup vtknetcdf_ncstrndup
#define nc_sync vtknetcdf_nc_sync
#define NC_sync vtknetcdf_NC_sync
#define nc__testurl vtknetcdf_nc__testurl
#define NC_testurl vtknetcdf_NC_testurl
#define nctypealignment vtknetcdf_nctypealignment
#define nctypelen vtknetcdf_nctypelen
#define ncuribuild vtknetcdf_ncuribuild
#define ncuridecode vtknetcdf_ncuridecode
#define ncuridecodepartial vtknetcdf_ncuridecodepartial
#define ncuriencodeonly vtknetcdf_ncuriencodeonly
#define ncuriencodeuserpwd vtknetcdf_ncuriencodeuserpwd
#define ncurifree vtknetcdf_ncurifree
#define ncurilookup vtknetcdf_ncurilookup
#define ncuriparse vtknetcdf_ncuriparse
#define ncuriquerylookup vtknetcdf_ncuriquerylookup
#define ncurisetprotocol vtknetcdf_ncurisetprotocol
#define ncurisetquery vtknetcdf_ncurisetquery
#define NC_urlmodel vtknetcdf_NC_urlmodel
#define nc_utf8_normalize vtknetcdf_nc_utf8_normalize
#define nc_utf8proc_category vtknetcdf_nc_utf8proc_category
#define nc_utf8proc_category_string vtknetcdf_nc_utf8proc_category_string
#define nc_utf8proc_charwidth vtknetcdf_nc_utf8proc_charwidth
#define nc_utf8proc_codepoint_valid vtknetcdf_nc_utf8proc_codepoint_valid
#define nc_utf8proc_combinations vtknetcdf_nc_utf8proc_combinations
#define nc_utf8proc_decompose vtknetcdf_nc_utf8proc_decompose
#define nc_utf8proc_decompose_char vtknetcdf_nc_utf8proc_decompose_char
#define nc_utf8proc_decompose_custom vtknetcdf_nc_utf8proc_decompose_custom
#define nc_utf8proc_encode_char vtknetcdf_nc_utf8proc_encode_char
#define nc_utf8proc_errmsg vtknetcdf_nc_utf8proc_errmsg
#define nc_utf8proc_get_property vtknetcdf_nc_utf8proc_get_property
#define nc_utf8proc_grapheme_break vtknetcdf_nc_utf8proc_grapheme_break
#define nc_utf8proc_grapheme_break_stateful vtknetcdf_nc_utf8proc_grapheme_break_stateful
#define nc_utf8proc_iterate vtknetcdf_nc_utf8proc_iterate
#define nc_utf8proc_map vtknetcdf_nc_utf8proc_map
#define nc_utf8proc_map_custom vtknetcdf_nc_utf8proc_map_custom
#define nc_utf8proc_NFC vtknetcdf_nc_utf8proc_NFC
#define nc_utf8proc_NFD vtknetcdf_nc_utf8proc_NFD
#define nc_utf8proc_NFKC vtknetcdf_nc_utf8proc_NFKC
#define nc_utf8proc_NFKD vtknetcdf_nc_utf8proc_NFKD
#define nc_utf8proc_normalize_utf32 vtknetcdf_nc_utf8proc_normalize_utf32
#define nc_utf8proc_properties vtknetcdf_nc_utf8proc_properties
#define nc_utf8proc_reencode vtknetcdf_nc_utf8proc_reencode
#define nc_utf8proc_sequences vtknetcdf_nc_utf8proc_sequences
#define nc_utf8proc_stage1table vtknetcdf_nc_utf8proc_stage1table
#define nc_utf8proc_stage2table vtknetcdf_nc_utf8proc_stage2table
#define nc_utf8proc_tolower vtknetcdf_nc_utf8proc_tolower
#define nc_utf8proc_totitle vtknetcdf_nc_utf8proc_totitle
#define nc_utf8proc_toupper vtknetcdf_nc_utf8proc_toupper
#define nc_utf8proc_utf8class vtknetcdf_nc_utf8proc_utf8class
#define nc_utf8proc_version vtknetcdf_nc_utf8proc_version
#define nc_utf8_to_utf16 vtknetcdf_nc_utf8_to_utf16
#define nc_utf8_validate vtknetcdf_nc_utf8_validate
#define nc_var_par_access vtknetcdf_nc_var_par_access
#define NC_var_shape vtknetcdf_NC_var_shape
#define ncx_getn_double_double vtknetcdf_ncx_getn_double_double
#define ncx_getn_double_float vtknetcdf_ncx_getn_double_float
#define ncx_getn_double_int vtknetcdf_ncx_getn_double_int
#define ncx_getn_double_long vtknetcdf_ncx_getn_double_long
#define ncx_getn_double_longlong vtknetcdf_ncx_getn_double_longlong
#define ncx_getn_double_schar vtknetcdf_ncx_getn_double_schar
#define ncx_getn_double_short vtknetcdf_ncx_getn_double_short
#define ncx_getn_double_uchar vtknetcdf_ncx_getn_double_uchar
#define ncx_getn_double_uint vtknetcdf_ncx_getn_double_uint
#define ncx_getn_double_ulonglong vtknetcdf_ncx_getn_double_ulonglong
#define ncx_getn_double_ushort vtknetcdf_ncx_getn_double_ushort
#define ncx_getn_float_double vtknetcdf_ncx_getn_float_double
#define ncx_getn_float_float vtknetcdf_ncx_getn_float_float
#define ncx_getn_float_int vtknetcdf_ncx_getn_float_int
#define ncx_getn_float_long vtknetcdf_ncx_getn_float_long
#define ncx_getn_float_longlong vtknetcdf_ncx_getn_float_longlong
#define ncx_getn_float_schar vtknetcdf_ncx_getn_float_schar
#define ncx_getn_float_short vtknetcdf_ncx_getn_float_short
#define ncx_getn_float_uchar vtknetcdf_ncx_getn_float_uchar
#define ncx_getn_float_uint vtknetcdf_ncx_getn_float_uint
#define ncx_getn_float_ulonglong vtknetcdf_ncx_getn_float_ulonglong
#define ncx_getn_float_ushort vtknetcdf_ncx_getn_float_ushort
#define ncx_getn_int_double vtknetcdf_ncx_getn_int_double
#define ncx_getn_int_float vtknetcdf_ncx_getn_int_float
#define ncx_getn_int_int vtknetcdf_ncx_getn_int_int
#define ncx_getn_int_long vtknetcdf_ncx_getn_int_long
#define ncx_getn_int_longlong vtknetcdf_ncx_getn_int_longlong
#define ncx_getn_int_schar vtknetcdf_ncx_getn_int_schar
#define ncx_getn_int_short vtknetcdf_ncx_getn_int_short
#define ncx_getn_int_uchar vtknetcdf_ncx_getn_int_uchar
#define ncx_getn_int_uint vtknetcdf_ncx_getn_int_uint
#define ncx_getn_int_ulonglong vtknetcdf_ncx_getn_int_ulonglong
#define ncx_getn_int_ushort vtknetcdf_ncx_getn_int_ushort
#define ncx_getn_longlong_double vtknetcdf_ncx_getn_longlong_double
#define ncx_getn_longlong_float vtknetcdf_ncx_getn_longlong_float
#define ncx_getn_longlong_int vtknetcdf_ncx_getn_longlong_int
#define ncx_getn_longlong_long vtknetcdf_ncx_getn_longlong_long
#define ncx_getn_longlong_longlong vtknetcdf_ncx_getn_longlong_longlong
#define ncx_getn_longlong_schar vtknetcdf_ncx_getn_longlong_schar
#define ncx_getn_longlong_short vtknetcdf_ncx_getn_longlong_short
#define ncx_getn_longlong_uchar vtknetcdf_ncx_getn_longlong_uchar
#define ncx_getn_longlong_uint vtknetcdf_ncx_getn_longlong_uint
#define ncx_getn_longlong_ulonglong vtknetcdf_ncx_getn_longlong_ulonglong
#define ncx_getn_longlong_ushort vtknetcdf_ncx_getn_longlong_ushort
#define ncx_getn_schar_double vtknetcdf_ncx_getn_schar_double
#define ncx_getn_schar_float vtknetcdf_ncx_getn_schar_float
#define ncx_getn_schar_int vtknetcdf_ncx_getn_schar_int
#define ncx_getn_schar_long vtknetcdf_ncx_getn_schar_long
#define ncx_getn_schar_longlong vtknetcdf_ncx_getn_schar_longlong
#define ncx_getn_schar_schar vtknetcdf_ncx_getn_schar_schar
#define ncx_getn_schar_short vtknetcdf_ncx_getn_schar_short
#define ncx_getn_schar_uchar vtknetcdf_ncx_getn_schar_uchar
#define ncx_getn_schar_uint vtknetcdf_ncx_getn_schar_uint
#define ncx_getn_schar_ulonglong vtknetcdf_ncx_getn_schar_ulonglong
#define ncx_getn_schar_ushort vtknetcdf_ncx_getn_schar_ushort
#define ncx_getn_short_double vtknetcdf_ncx_getn_short_double
#define ncx_getn_short_float vtknetcdf_ncx_getn_short_float
#define ncx_getn_short_int vtknetcdf_ncx_getn_short_int
#define ncx_getn_short_long vtknetcdf_ncx_getn_short_long
#define ncx_getn_short_longlong vtknetcdf_ncx_getn_short_longlong
#define ncx_getn_short_schar vtknetcdf_ncx_getn_short_schar
#define ncx_getn_short_short vtknetcdf_ncx_getn_short_short
#define ncx_getn_short_uchar vtknetcdf_ncx_getn_short_uchar
#define ncx_getn_short_uint vtknetcdf_ncx_getn_short_uint
#define ncx_getn_short_ulonglong vtknetcdf_ncx_getn_short_ulonglong
#define ncx_getn_short_ushort vtknetcdf_ncx_getn_short_ushort
#define ncx_getn_text vtknetcdf_ncx_getn_text
#define ncx_getn_uchar_double vtknetcdf_ncx_getn_uchar_double
#define ncx_getn_uchar_float vtknetcdf_ncx_getn_uchar_float
#define ncx_getn_uchar_int vtknetcdf_ncx_getn_uchar_int
#define ncx_getn_uchar_long vtknetcdf_ncx_getn_uchar_long
#define ncx_getn_uchar_longlong vtknetcdf_ncx_getn_uchar_longlong
#define ncx_getn_uchar_schar vtknetcdf_ncx_getn_uchar_schar
#define ncx_getn_uchar_short vtknetcdf_ncx_getn_uchar_short
#define ncx_getn_uchar_uchar vtknetcdf_ncx_getn_uchar_uchar
#define ncx_getn_uchar_uint vtknetcdf_ncx_getn_uchar_uint
#define ncx_getn_uchar_ulonglong vtknetcdf_ncx_getn_uchar_ulonglong
#define ncx_getn_uchar_ushort vtknetcdf_ncx_getn_uchar_ushort
#define ncx_getn_uint_double vtknetcdf_ncx_getn_uint_double
#define ncx_getn_uint_float vtknetcdf_ncx_getn_uint_float
#define ncx_getn_uint_int vtknetcdf_ncx_getn_uint_int
#define ncx_getn_uint_long vtknetcdf_ncx_getn_uint_long
#define ncx_getn_uint_longlong vtknetcdf_ncx_getn_uint_longlong
#define ncx_getn_uint_schar vtknetcdf_ncx_getn_uint_schar
#define ncx_getn_uint_short vtknetcdf_ncx_getn_uint_short
#define ncx_getn_uint_uchar vtknetcdf_ncx_getn_uint_uchar
#define ncx_getn_uint_uint vtknetcdf_ncx_getn_uint_uint
#define ncx_getn_uint_ulonglong vtknetcdf_ncx_getn_uint_ulonglong
#define ncx_getn_uint_ushort vtknetcdf_ncx_getn_uint_ushort
#define ncx_getn_ulonglong_double vtknetcdf_ncx_getn_ulonglong_double
#define ncx_getn_ulonglong_float vtknetcdf_ncx_getn_ulonglong_float
#define ncx_getn_ulonglong_int vtknetcdf_ncx_getn_ulonglong_int
#define ncx_getn_ulonglong_long vtknetcdf_ncx_getn_ulonglong_long
#define ncx_getn_ulonglong_longlong vtknetcdf_ncx_getn_ulonglong_longlong
#define ncx_getn_ulonglong_schar vtknetcdf_ncx_getn_ulonglong_schar
#define ncx_getn_ulonglong_short vtknetcdf_ncx_getn_ulonglong_short
#define ncx_getn_ulonglong_uchar vtknetcdf_ncx_getn_ulonglong_uchar
#define ncx_getn_ulonglong_uint vtknetcdf_ncx_getn_ulonglong_uint
#define ncx_getn_ulonglong_ulonglong vtknetcdf_ncx_getn_ulonglong_ulonglong
#define ncx_getn_ulonglong_ushort vtknetcdf_ncx_getn_ulonglong_ushort
#define ncx_getn_ushort_double vtknetcdf_ncx_getn_ushort_double
#define ncx_getn_ushort_float vtknetcdf_ncx_getn_ushort_float
#define ncx_getn_ushort_int vtknetcdf_ncx_getn_ushort_int
#define ncx_getn_ushort_long vtknetcdf_ncx_getn_ushort_long
#define ncx_getn_ushort_longlong vtknetcdf_ncx_getn_ushort_longlong
#define ncx_getn_ushort_schar vtknetcdf_ncx_getn_ushort_schar
#define ncx_getn_ushort_short vtknetcdf_ncx_getn_ushort_short
#define ncx_getn_ushort_uchar vtknetcdf_ncx_getn_ushort_uchar
#define ncx_getn_ushort_uint vtknetcdf_ncx_getn_ushort_uint
#define ncx_getn_ushort_ulonglong vtknetcdf_ncx_getn_ushort_ulonglong
#define ncx_getn_ushort_ushort vtknetcdf_ncx_getn_ushort_ushort
#define ncx_getn_void vtknetcdf_ncx_getn_void
#define ncx_get_off_t vtknetcdf_ncx_get_off_t
#define ncx_get_size_t vtknetcdf_ncx_get_size_t
#define ncx_get_uint32 vtknetcdf_ncx_get_uint32
#define ncx_get_uint64 vtknetcdf_ncx_get_uint64
#define ncx_howmany vtknetcdf_ncx_howmany
#define ncx_len_NC vtknetcdf_ncx_len_NC
#define ncx_pad_getn_schar_double vtknetcdf_ncx_pad_getn_schar_double
#define ncx_pad_getn_schar_float vtknetcdf_ncx_pad_getn_schar_float
#define ncx_pad_getn_schar_int vtknetcdf_ncx_pad_getn_schar_int
#define ncx_pad_getn_schar_long vtknetcdf_ncx_pad_getn_schar_long
#define ncx_pad_getn_schar_longlong vtknetcdf_ncx_pad_getn_schar_longlong
#define ncx_pad_getn_schar_schar vtknetcdf_ncx_pad_getn_schar_schar
#define ncx_pad_getn_schar_short vtknetcdf_ncx_pad_getn_schar_short
#define ncx_pad_getn_schar_uchar vtknetcdf_ncx_pad_getn_schar_uchar
#define ncx_pad_getn_schar_uint vtknetcdf_ncx_pad_getn_schar_uint
#define ncx_pad_getn_schar_ulonglong vtknetcdf_ncx_pad_getn_schar_ulonglong
#define ncx_pad_getn_schar_ushort vtknetcdf_ncx_pad_getn_schar_ushort
#define ncx_pad_getn_short_double vtknetcdf_ncx_pad_getn_short_double
#define ncx_pad_getn_short_float vtknetcdf_ncx_pad_getn_short_float
#define ncx_pad_getn_short_int vtknetcdf_ncx_pad_getn_short_int
#define ncx_pad_getn_short_long vtknetcdf_ncx_pad_getn_short_long
#define ncx_pad_getn_short_longlong vtknetcdf_ncx_pad_getn_short_longlong
#define ncx_pad_getn_short_schar vtknetcdf_ncx_pad_getn_short_schar
#define ncx_pad_getn_short_short vtknetcdf_ncx_pad_getn_short_short
#define ncx_pad_getn_short_uchar vtknetcdf_ncx_pad_getn_short_uchar
#define ncx_pad_getn_short_uint vtknetcdf_ncx_pad_getn_short_uint
#define ncx_pad_getn_short_ulonglong vtknetcdf_ncx_pad_getn_short_ulonglong
#define ncx_pad_getn_short_ushort vtknetcdf_ncx_pad_getn_short_ushort
#define ncx_pad_getn_text vtknetcdf_ncx_pad_getn_text
#define ncx_pad_getn_uchar_double vtknetcdf_ncx_pad_getn_uchar_double
#define ncx_pad_getn_uchar_float vtknetcdf_ncx_pad_getn_uchar_float
#define ncx_pad_getn_uchar_int vtknetcdf_ncx_pad_getn_uchar_int
#define ncx_pad_getn_uchar_long vtknetcdf_ncx_pad_getn_uchar_long
#define ncx_pad_getn_uchar_longlong vtknetcdf_ncx_pad_getn_uchar_longlong
#define ncx_pad_getn_uchar_schar vtknetcdf_ncx_pad_getn_uchar_schar
#define ncx_pad_getn_uchar_short vtknetcdf_ncx_pad_getn_uchar_short
#define ncx_pad_getn_uchar_uchar vtknetcdf_ncx_pad_getn_uchar_uchar
#define ncx_pad_getn_uchar_uint vtknetcdf_ncx_pad_getn_uchar_uint
#define ncx_pad_getn_uchar_ulonglong vtknetcdf_ncx_pad_getn_uchar_ulonglong
#define ncx_pad_getn_uchar_ushort vtknetcdf_ncx_pad_getn_uchar_ushort
#define ncx_pad_getn_ushort_double vtknetcdf_ncx_pad_getn_ushort_double
#define ncx_pad_getn_ushort_float vtknetcdf_ncx_pad_getn_ushort_float
#define ncx_pad_getn_ushort_int vtknetcdf_ncx_pad_getn_ushort_int
#define ncx_pad_getn_ushort_long vtknetcdf_ncx_pad_getn_ushort_long
#define ncx_pad_getn_ushort_longlong vtknetcdf_ncx_pad_getn_ushort_longlong
#define ncx_pad_getn_ushort_schar vtknetcdf_ncx_pad_getn_ushort_schar
#define ncx_pad_getn_ushort_short vtknetcdf_ncx_pad_getn_ushort_short
#define ncx_pad_getn_ushort_uchar vtknetcdf_ncx_pad_getn_ushort_uchar
#define ncx_pad_getn_ushort_uint vtknetcdf_ncx_pad_getn_ushort_uint
#define ncx_pad_getn_ushort_ulonglong vtknetcdf_ncx_pad_getn_ushort_ulonglong
#define ncx_pad_getn_ushort_ushort vtknetcdf_ncx_pad_getn_ushort_ushort
#define ncx_pad_getn_void vtknetcdf_ncx_pad_getn_void
#define ncx_pad_putn_schar_double vtknetcdf_ncx_pad_putn_schar_double
#define ncx_pad_putn_schar_float vtknetcdf_ncx_pad_putn_schar_float
#define ncx_pad_putn_schar_int vtknetcdf_ncx_pad_putn_schar_int
#define ncx_pad_putn_schar_long vtknetcdf_ncx_pad_putn_schar_long
#define ncx_pad_putn_schar_longlong vtknetcdf_ncx_pad_putn_schar_longlong
#define ncx_pad_putn_schar_schar vtknetcdf_ncx_pad_putn_schar_schar
#define ncx_pad_putn_schar_short vtknetcdf_ncx_pad_putn_schar_short
#define ncx_pad_putn_schar_uchar vtknetcdf_ncx_pad_putn_schar_uchar
#define ncx_pad_putn_schar_uint vtknetcdf_ncx_pad_putn_schar_uint
#define ncx_pad_putn_schar_ulonglong vtknetcdf_ncx_pad_putn_schar_ulonglong
#define ncx_pad_putn_schar_ushort vtknetcdf_ncx_pad_putn_schar_ushort
#define ncx_pad_putn_short_double vtknetcdf_ncx_pad_putn_short_double
#define ncx_pad_putn_short_float vtknetcdf_ncx_pad_putn_short_float
#define ncx_pad_putn_short_int vtknetcdf_ncx_pad_putn_short_int
#define ncx_pad_putn_short_long vtknetcdf_ncx_pad_putn_short_long
#define ncx_pad_putn_short_longlong vtknetcdf_ncx_pad_putn_short_longlong
#define ncx_pad_putn_short_schar vtknetcdf_ncx_pad_putn_short_schar
#define ncx_pad_putn_short_short vtknetcdf_ncx_pad_putn_short_short
#define ncx_pad_putn_short_uchar vtknetcdf_ncx_pad_putn_short_uchar
#define ncx_pad_putn_short_uint vtknetcdf_ncx_pad_putn_short_uint
#define ncx_pad_putn_short_ulonglong vtknetcdf_ncx_pad_putn_short_ulonglong
#define ncx_pad_putn_short_ushort vtknetcdf_ncx_pad_putn_short_ushort
#define ncx_pad_putn_text vtknetcdf_ncx_pad_putn_text
#define ncx_pad_putn_uchar_double vtknetcdf_ncx_pad_putn_uchar_double
#define ncx_pad_putn_uchar_float vtknetcdf_ncx_pad_putn_uchar_float
#define ncx_pad_putn_uchar_int vtknetcdf_ncx_pad_putn_uchar_int
#define ncx_pad_putn_uchar_long vtknetcdf_ncx_pad_putn_uchar_long
#define ncx_pad_putn_uchar_longlong vtknetcdf_ncx_pad_putn_uchar_longlong
#define ncx_pad_putn_uchar_schar vtknetcdf_ncx_pad_putn_uchar_schar
#define ncx_pad_putn_uchar_short vtknetcdf_ncx_pad_putn_uchar_short
#define ncx_pad_putn_uchar_uchar vtknetcdf_ncx_pad_putn_uchar_uchar
#define ncx_pad_putn_uchar_uint vtknetcdf_ncx_pad_putn_uchar_uint
#define ncx_pad_putn_uchar_ulonglong vtknetcdf_ncx_pad_putn_uchar_ulonglong
#define ncx_pad_putn_uchar_ushort vtknetcdf_ncx_pad_putn_uchar_ushort
#define ncx_pad_putn_ushort_double vtknetcdf_ncx_pad_putn_ushort_double
#define ncx_pad_putn_ushort_float vtknetcdf_ncx_pad_putn_ushort_float
#define ncx_pad_putn_ushort_int vtknetcdf_ncx_pad_putn_ushort_int
#define ncx_pad_putn_ushort_long vtknetcdf_ncx_pad_putn_ushort_long
#define ncx_pad_putn_ushort_longlong vtknetcdf_ncx_pad_putn_ushort_longlong
#define ncx_pad_putn_ushort_schar vtknetcdf_ncx_pad_putn_ushort_schar
#define ncx_pad_putn_ushort_short vtknetcdf_ncx_pad_putn_ushort_short
#define ncx_pad_putn_ushort_uchar vtknetcdf_ncx_pad_putn_ushort_uchar
#define ncx_pad_putn_ushort_uint vtknetcdf_ncx_pad_putn_ushort_uint
#define ncx_pad_putn_ushort_ulonglong vtknetcdf_ncx_pad_putn_ushort_ulonglong
#define ncx_pad_putn_ushort_ushort vtknetcdf_ncx_pad_putn_ushort_ushort
#define ncx_pad_putn_void vtknetcdf_ncx_pad_putn_void
#define ncx_put_NC vtknetcdf_ncx_put_NC
#define ncx_putn_double_double vtknetcdf_ncx_putn_double_double
#define ncx_putn_double_float vtknetcdf_ncx_putn_double_float
#define ncx_putn_double_int vtknetcdf_ncx_putn_double_int
#define ncx_putn_double_long vtknetcdf_ncx_putn_double_long
#define ncx_putn_double_longlong vtknetcdf_ncx_putn_double_longlong
#define ncx_putn_double_schar vtknetcdf_ncx_putn_double_schar
#define ncx_putn_double_short vtknetcdf_ncx_putn_double_short
#define ncx_putn_double_uchar vtknetcdf_ncx_putn_double_uchar
#define ncx_putn_double_uint vtknetcdf_ncx_putn_double_uint
#define ncx_putn_double_ulonglong vtknetcdf_ncx_putn_double_ulonglong
#define ncx_putn_double_ushort vtknetcdf_ncx_putn_double_ushort
#define ncx_putn_float_double vtknetcdf_ncx_putn_float_double
#define ncx_putn_float_float vtknetcdf_ncx_putn_float_float
#define ncx_putn_float_int vtknetcdf_ncx_putn_float_int
#define ncx_putn_float_long vtknetcdf_ncx_putn_float_long
#define ncx_putn_float_longlong vtknetcdf_ncx_putn_float_longlong
#define ncx_putn_float_schar vtknetcdf_ncx_putn_float_schar
#define ncx_putn_float_short vtknetcdf_ncx_putn_float_short
#define ncx_putn_float_uchar vtknetcdf_ncx_putn_float_uchar
#define ncx_putn_float_uint vtknetcdf_ncx_putn_float_uint
#define ncx_putn_float_ulonglong vtknetcdf_ncx_putn_float_ulonglong
#define ncx_putn_float_ushort vtknetcdf_ncx_putn_float_ushort
#define ncx_putn_int_double vtknetcdf_ncx_putn_int_double
#define ncx_putn_int_float vtknetcdf_ncx_putn_int_float
#define ncx_putn_int_int vtknetcdf_ncx_putn_int_int
#define ncx_putn_int_long vtknetcdf_ncx_putn_int_long
#define ncx_putn_int_longlong vtknetcdf_ncx_putn_int_longlong
#define ncx_putn_int_schar vtknetcdf_ncx_putn_int_schar
#define ncx_putn_int_short vtknetcdf_ncx_putn_int_short
#define ncx_putn_int_uchar vtknetcdf_ncx_putn_int_uchar
#define ncx_putn_int_uint vtknetcdf_ncx_putn_int_uint
#define ncx_putn_int_ulonglong vtknetcdf_ncx_putn_int_ulonglong
#define ncx_putn_int_ushort vtknetcdf_ncx_putn_int_ushort
#define ncx_putn_longlong_double vtknetcdf_ncx_putn_longlong_double
#define ncx_putn_longlong_float vtknetcdf_ncx_putn_longlong_float
#define ncx_putn_longlong_int vtknetcdf_ncx_putn_longlong_int
#define ncx_putn_longlong_long vtknetcdf_ncx_putn_longlong_long
#define ncx_putn_longlong_longlong vtknetcdf_ncx_putn_longlong_longlong
#define ncx_putn_longlong_schar vtknetcdf_ncx_putn_longlong_schar
#define ncx_putn_longlong_short vtknetcdf_ncx_putn_longlong_short
#define ncx_putn_longlong_uchar vtknetcdf_ncx_putn_longlong_uchar
#define ncx_putn_longlong_uint vtknetcdf_ncx_putn_longlong_uint
#define ncx_putn_longlong_ulonglong vtknetcdf_ncx_putn_longlong_ulonglong
#define ncx_putn_longlong_ushort vtknetcdf_ncx_putn_longlong_ushort
#define ncx_putn_schar_double vtknetcdf_ncx_putn_schar_double
#define ncx_putn_schar_float vtknetcdf_ncx_putn_schar_float
#define ncx_putn_schar_int vtknetcdf_ncx_putn_schar_int
#define ncx_putn_schar_long vtknetcdf_ncx_putn_schar_long
#define ncx_putn_schar_longlong vtknetcdf_ncx_putn_schar_longlong
#define ncx_putn_schar_schar vtknetcdf_ncx_putn_schar_schar
#define ncx_putn_schar_short vtknetcdf_ncx_putn_schar_short
#define ncx_putn_schar_uchar vtknetcdf_ncx_putn_schar_uchar
#define ncx_putn_schar_uint vtknetcdf_ncx_putn_schar_uint
#define ncx_putn_schar_ulonglong vtknetcdf_ncx_putn_schar_ulonglong
#define ncx_putn_schar_ushort vtknetcdf_ncx_putn_schar_ushort
#define ncx_putn_short_double vtknetcdf_ncx_putn_short_double
#define ncx_putn_short_float vtknetcdf_ncx_putn_short_float
#define ncx_putn_short_int vtknetcdf_ncx_putn_short_int
#define ncx_putn_short_long vtknetcdf_ncx_putn_short_long
#define ncx_putn_short_longlong vtknetcdf_ncx_putn_short_longlong
#define ncx_putn_short_schar vtknetcdf_ncx_putn_short_schar
#define ncx_putn_short_short vtknetcdf_ncx_putn_short_short
#define ncx_putn_short_uchar vtknetcdf_ncx_putn_short_uchar
#define ncx_putn_short_uint vtknetcdf_ncx_putn_short_uint
#define ncx_putn_short_ulonglong vtknetcdf_ncx_putn_short_ulonglong
#define ncx_putn_short_ushort vtknetcdf_ncx_putn_short_ushort
#define ncx_putn_text vtknetcdf_ncx_putn_text
#define ncx_putn_uchar_double vtknetcdf_ncx_putn_uchar_double
#define ncx_putn_uchar_float vtknetcdf_ncx_putn_uchar_float
#define ncx_putn_uchar_int vtknetcdf_ncx_putn_uchar_int
#define ncx_putn_uchar_long vtknetcdf_ncx_putn_uchar_long
#define ncx_putn_uchar_longlong vtknetcdf_ncx_putn_uchar_longlong
#define ncx_putn_uchar_schar vtknetcdf_ncx_putn_uchar_schar
#define ncx_putn_uchar_short vtknetcdf_ncx_putn_uchar_short
#define ncx_putn_uchar_uchar vtknetcdf_ncx_putn_uchar_uchar
#define ncx_putn_uchar_uint vtknetcdf_ncx_putn_uchar_uint
#define ncx_putn_uchar_ulonglong vtknetcdf_ncx_putn_uchar_ulonglong
#define ncx_putn_uchar_ushort vtknetcdf_ncx_putn_uchar_ushort
#define ncx_putn_uint_double vtknetcdf_ncx_putn_uint_double
#define ncx_putn_uint_float vtknetcdf_ncx_putn_uint_float
#define ncx_putn_uint_int vtknetcdf_ncx_putn_uint_int
#define ncx_putn_uint_long vtknetcdf_ncx_putn_uint_long
#define ncx_putn_uint_longlong vtknetcdf_ncx_putn_uint_longlong
#define ncx_putn_uint_schar vtknetcdf_ncx_putn_uint_schar
#define ncx_putn_uint_short vtknetcdf_ncx_putn_uint_short
#define ncx_putn_uint_uchar vtknetcdf_ncx_putn_uint_uchar
#define ncx_putn_uint_uint vtknetcdf_ncx_putn_uint_uint
#define ncx_putn_uint_ulonglong vtknetcdf_ncx_putn_uint_ulonglong
#define ncx_putn_uint_ushort vtknetcdf_ncx_putn_uint_ushort
#define ncx_putn_ulonglong_double vtknetcdf_ncx_putn_ulonglong_double
#define ncx_putn_ulonglong_float vtknetcdf_ncx_putn_ulonglong_float
#define ncx_putn_ulonglong_int vtknetcdf_ncx_putn_ulonglong_int
#define ncx_putn_ulonglong_long vtknetcdf_ncx_putn_ulonglong_long
#define ncx_putn_ulonglong_longlong vtknetcdf_ncx_putn_ulonglong_longlong
#define ncx_putn_ulonglong_schar vtknetcdf_ncx_putn_ulonglong_schar
#define ncx_putn_ulonglong_short vtknetcdf_ncx_putn_ulonglong_short
#define ncx_putn_ulonglong_uchar vtknetcdf_ncx_putn_ulonglong_uchar
#define ncx_putn_ulonglong_uint vtknetcdf_ncx_putn_ulonglong_uint
#define ncx_putn_ulonglong_ulonglong vtknetcdf_ncx_putn_ulonglong_ulonglong
#define ncx_putn_ulonglong_ushort vtknetcdf_ncx_putn_ulonglong_ushort
#define ncx_putn_ushort_double vtknetcdf_ncx_putn_ushort_double
#define ncx_putn_ushort_float vtknetcdf_ncx_putn_ushort_float
#define ncx_putn_ushort_int vtknetcdf_ncx_putn_ushort_int
#define ncx_putn_ushort_long vtknetcdf_ncx_putn_ushort_long
#define ncx_putn_ushort_longlong vtknetcdf_ncx_putn_ushort_longlong
#define ncx_putn_ushort_schar vtknetcdf_ncx_putn_ushort_schar
#define ncx_putn_ushort_short vtknetcdf_ncx_putn_ushort_short
#define ncx_putn_ushort_uchar vtknetcdf_ncx_putn_ushort_uchar
#define ncx_putn_ushort_uint vtknetcdf_ncx_putn_ushort_uint
#define ncx_putn_ushort_ulonglong vtknetcdf_ncx_putn_ushort_ulonglong
#define ncx_putn_ushort_ushort vtknetcdf_ncx_putn_ushort_ushort
#define ncx_putn_void vtknetcdf_ncx_putn_void
#define ncx_put_off_t vtknetcdf_ncx_put_off_t
#define ncx_put_size_t vtknetcdf_ncx_put_size_t
#define ncx_put_uint32 vtknetcdf_ncx_put_uint32
#define ncx_put_uint64 vtknetcdf_ncx_put_uint64
#define ncx_szof vtknetcdf_ncx_szof
#define new_NC vtknetcdf_new_NC
#define new_NC_string vtknetcdf_new_NC_string
#define new_x_NC_attr vtknetcdf_new_x_NC_attr
#define new_x_NC_dim vtknetcdf_new_x_NC_dim
#define new_x_NC_var vtknetcdf_new_x_NC_var
#define posixio_create vtknetcdf_posixio_create
#define posixio_open vtknetcdf_posixio_open
#define read_numrecs vtknetcdf_read_numrecs
#define rec_detach_scales vtknetcdf_rec_detach_scales
#define rec_reattach_scales vtknetcdf_rec_reattach_scales
#define reportobject vtknetcdf_reportobject
#define reportopenobjects vtknetcdf_reportopenobjects
#define set_NC_string vtknetcdf_set_NC_string
#define write_numrecs vtknetcdf_write_numrecs
#define ncindexadd vtknetcdf_ncindexadd
#define ncindexcount vtknetcdf_ncindexcount
#define ncindexdup vtknetcdf_ncindexdup
#define ncindexfind vtknetcdf_ncindexfind
#define ncindexfree vtknetcdf_ncindexfree
#define ncindexidel vtknetcdf_ncindexidel
#define ncindexith vtknetcdf_ncindexith
#define ncindexlookup vtknetcdf_ncindexlookup
#define ncindexnew vtknetcdf_ncindexnew
#define ncindexrebuild vtknetcdf_ncindexrebuild
#define ncindexset vtknetcdf_ncindexset
#define ncindexverify vtknetcdf_ncindexverify
#define NC_infermodel vtknetcdf_NC_infermodel
#define nc__testurl vtknetcdf_nc__testurl
#define printindex vtknetcdf_printindex
#define printindexlist vtknetcdf_printindexlist
#define printindexmap vtknetcdf_printindexmap
#define NC_crc32 vtknetcdf_NC_crc32

#endif
