
#ifndef VTKARROWGLYPHFILTER_EXPORT_H
#define VTKARROWGLYPHFILTER_EXPORT_H

#ifdef VTKARROWGLYPHFILTER_STATIC_DEFINE
#  define VTKARROWGLYPHFILTER_EXPORT
#  define VTKARROWGLYPHFILTER_NO_EXPORT
#else
#  ifndef VTKARROWGLYPHFILTER_EXPORT
#    ifdef ArrowGlyphFilter_EXPORTS
        /* We are building this library */
#      define VTKARROWGLYPHFILTER_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKARROWGLYPHFILTER_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKARROWGLYPHFILTER_NO_EXPORT
#    define VTKARROWGLYPHFILTER_NO_EXPORT 
#  endif
#endif

#ifndef VTKARROWGLYPHFILTER_DEPRECATED
#  define VTKARROWGLYPHFILTER_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKARROWGLYPHFILTER_DEPRECATED_EXPORT
#  define VTKARROWGLYPHFILTER_DEPRECATED_EXPORT VTKARROWGLYPHFILTER_EXPORT VTKARROWGLYPHFILTER_DEPRECATED
#endif

#ifndef VTKARROWGLYPHFILTER_DEPRECATED_NO_EXPORT
#  define VTKARROWGLYPHFILTER_DEPRECATED_NO_EXPORT VTKARROWGLYPHFILTER_NO_EXPORT VTKARROWGLYPHFILTER_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKARROWGLYPHFILTER_NO_DEPRECATED
#    define VTKARROWGLYPHFILTER_NO_DEPRECATED
#  endif
#endif

#endif /* VTKARROWGLYPHFILTER_EXPORT_H */
