/*=========================================================================

  Program:   ParaView
  Module:    vtkPVConfig.h.in

  Copyright (c) Kitware, Inc.
  All rights reserved.
  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

#ifndef vtkPVConfig_h
#define vtkPVConfig_h

#define PARAVIEW_USE_QTHELP

#define PARAVIEW_VERSION_MAJOR 5
#define PARAVIEW_VERSION_MINOR 8
#define PARAVIEW_VERSION_PATCH 0
#define PARAVIEW_VERSION "5.8"
#define PARAVIEW_VERSION_FULL "5.8.0-RC3"

#endif
